/*
 * Copyright 2000-2001 by Jean-Claude Christophe <jch@oleane.net>
 *                                               <jch@bsdfr.org>
 *
 * Home page:  http://www.bsdfr.org/projects/tac_bsd/
 *
 * conf.c  Some functions for parse configuration file
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>

#include "tac_bsd.h"
#include "conf.h"

/* 
 * global part of configuration 
 */

int  slog;          /* 0: syslog disable; 1: syslog enable */
int  slog_flty;     /* the syslog facility */
int  slog_lvl;      /* the syslog level */
int  _syslog;       /* the definitive syslog (facility | level) */
char *premotd;      /* file to display before asking username */
char *motdpass;     /* file to display when authentication succeded */
char *motdfail;     /* file to display when authentication failed */
char *shell;        /* list of shell to use */
int  maxtry;        /* you have 3 chance by default; */
int  maxsession;    /* sessions maximum by default */
int  maxlogintime;  /* timeout for each login attempt (0 will disable it) */
char *directory;    /* directory to use when logged in */
char *reject_user;  /* file containing all denied users */
int  passwd_care;   /* TRUE: we take care about the the /etc/passwd file, FALSE: ignore it */
int  permit_uid0;   /* TRUE: all local users with uid0 can log into the system */

char *server;       /* one of the server we'll connect to */
int server_timeout; /* connection timeout to the server */
u_char serv_count;  /* number of server defined in the configuration file */


// char check;         /* TRUE: check the config file */

struct s_options {
  char *options;
  char seq;
} line_conf[] = {{"server ",           1}, \
		 {"null",              2}, \
		 {"syslog ",           3}, \
		 {"syslog_facility ",  4}, \
		 {"syslog_level ",     5}, \
		 {"premotd ",          6}, \
		 {"motdpass ",         7}, \
		 {"motdfail ",         8}, \
		 {"shell ",            9}, \
		 {"maxtry ",          10}, \
		 {"maxsession ",      11}, \
		 {"directory ",       12}, \
		 {"umask ",           13}, \
		 {"passwd_care ",     14}, \
		 {"maxlogintime ",    15}, \
		 {"uid ",             16}, \
		 {"gid ",             17}, \
		 {"server_timeout ",  18}, \
		 {"reject_user ",     19}, \
		 {"null",             20}, \
		 {"permit_uid0 ",     21}};

/* 
 * syslog part of configuration 
 */
struct s_slog_flty {
  char *facility_in_conf;
  int facility;
} sslog_flty[] = {{"local0", LOG_LOCAL0}, \
		  {"local1", LOG_LOCAL1}, \
		  {"local2", LOG_LOCAL2}, \
		  {"local3", LOG_LOCAL3}, \
		  {"local4", LOG_LOCAL4}, \
		  {"local5", LOG_LOCAL5}, \
		  {"local6", LOG_LOCAL6}, \
		  {"local7", LOG_LOCAL7}};

struct s_slog_lvl {
  char *level_in_conf;
  int level;
} sslog_lvl[] = {{"emerg",   LOG_EMERG},   \
		 {"alert",   LOG_ALERT},   \
		 {"crit",    LOG_CRIT},    \
		 {"err",     LOG_ERR},     \
		 {"warning", LOG_WARNING}, \
		 {"notice",  LOG_NOTICE},  \
		 {"info",    LOG_INFO},    \
		 {"debug",   LOG_DEBUG}};

char *
set_conf_var(buf/*, check*/)
    char *buf;
//    char check;
{
    char *var;

/*    if (!var) {*/
    var = (char *)malloc(strlen(buf) + 1);
    strncpy(var, buf, strlen(buf));
    var[strlen(buf) - 1] = '\0';
    if (file_exist(var) < 0) {
	free(var);
	var = NULL;
    }
/*    } else if (check)
	printf("%s keyword already define\n", var);*/

    return(var);
}

int
read_conf(file)
    char *file;
{
    FILE *fd;     /* Our configuration file descriptor */
    char c[256];  /* it will contains lines of the configuration file */
    
    char *buf_line = (char *)malloc(256);
    int i;
    
    if (!(fd = fopen(file, "r")))
	return(-1); /* we can't open the configuration file */
    
    /* init some variables */
    maxtry = 3;
    maxsession = 3;
    maxlogintime = 300;
    server = premotd = motdpass = motdfail = directory = reject_user = NULL;
    server_timeout = 0;
    
    /* fill default variable */
    slog = 0;               /* don't use syslog by default */
    slog_flty = 8;          /* facility of syslog by default: local7 */
    slog_lvl = 7;           /* level of syslog by default: info */
    
    while (fgets(c, 256, fd)) {
	if (c[0] == '#') /* we ignore comment lines */
	    continue;
	if (!strncmp("DIE", c, 3)) { /* no quick start */
	    fprintf(stderr, "You have to make your configuration in the "CONF_FILE" file. Fatal.\n");
	    exit(-1);
	}
	for (i = 0; i < MAX_OPTIONSLINE; i++)
	    if (!strncmp(line_conf[i].options, c, strlen(line_conf[i].options))) {
		buf_line = c + strlen(line_conf[i].options);
		switch (line_conf[i].seq) {
		case 1: /* server */
		    server = (char *)malloc(strlen(buf_line)+1);
		    strncpy(server, buf_line, strlen(buf_line)+1);
		    {
			char *word, *brkt;
			
			/* name */
			word = strtok_r(server, " ", &brkt);
			if (word)
			    strncpy(tac_serv[serv_count].name, word, strlen(word));

			/* addr */
			word = strtok_r(NULL, " ", &brkt);
			if (word)
			    strncpy(tac_serv[serv_count].addr, word, strlen(word));

			/* key */
			word = strtok_r(NULL, " ", &brkt);
			word = strtok_r(NULL, " ", &brkt);
			if (word) {
			    strncpy(tac_serv[serv_count].key, word, strlen(word));
			    tac_serv[serv_count].key[strlen(word) - 1] = '\0';
			} else
			    tac_serv[serv_count].key[0] = '\0';

			serv_count++;
		    }
		    break;
		case 2: /* secret keyword obsolete */
		    break;
		case 3: /* syslog */
		    slog = (!strncmp(buf_line, "off", 3)) ? 0 : 1;
		    break;
		case 4: /* syslog_facility */
		{
		    int j;
		    for (j = 0; j < 8; j++)
			if (!strncmp(buf_line, sslog_flty[j].facility_in_conf, strlen(buf_line) - 1))
			    slog_flty = sslog_flty[j].facility;
		}
		break;
		case 5: /* syslog_level */
		{
		    int j;
		    for (j = 0; j < 8; j++)
			if (!strncmp(buf_line, sslog_lvl[j].level_in_conf, strlen(buf_line) - 1))
			    slog_lvl = sslog_lvl[j].level;
		}
		break;
		case 6: /* premotd */
		    premotd = set_conf_var(buf_line);
		    break;
		case 7: /* motdpass */
		    motdpass = set_conf_var(buf_line);
		    break;
		case 8: /* motdfail */
		    motdfail = set_conf_var(buf_line);
		    break;
		case 9: /* shell */
		    shell = set_conf_var(buf_line);
		    break;
		case 10: /* maxtry */
		    maxtry = atoi(buf_line);
		    break;
		case 11: /* maxsession */
		    maxsession = atoi(buf_line);
		    break;
		case 12: /* directory */
		    if (!directory) {
		        directory = (char *)malloc(strlen(buf_line) + 1);
			strncpy(directory, buf_line, strlen(buf_line));
			directory[strlen(buf_line) - 1] = '\0';
		    }
		    break;
		case 13: /* umask */
		    /* not implemented yet */
		    break;
		case 14: /* passwd_care */
		    passwd_care = (!strncmp(buf_line, "off", 3)) ? 0 : 1;
		    break;
		case 15: /* maxlogintime */
		    maxlogintime = atoi(buf_line);
		    break;
		case 16: /* uid */
		    uid_default = atoi(buf_line);
		    break;
		case 17: /* gid */
		    gid_default = atoi(buf_line);
		    break;
		case 18: /* server_timeout */
		    server_timeout = atoi(buf_line);
		    if (server_timeout > 30)
			server_timeout = 0;
		    break;
		case 19: /* reject_user */
		    reject_user = set_conf_var(buf_line);
		    break;
		case 20: /* group obsolete */
		  break;
		case 21: /* permit_uid0 */
		    permit_uid0 = (!strncmp(buf_line, "off", 3)) ? 0 : 1;
		  break;
		default:
		    break;
		}
	    }
    } /* while (.. */
    
    /* did we find any server in the configuration file ? */
    if ((tac_serv[0].name == '\0') && !passwd_care) {
	fprintf(stderr, "No server defined in the configuration file '%s'. Goodbye.\n", file);
	exit(-1);
    }

    /* did we find a shell path ? */
    if ((!shell) && (passwd_care == FALSE)) {
	fprintf(stderr, "No valid shell defined in the configuration file '%s'. Goodbye.\n", file);
	exit(-1);
    }

    /* are uid and gid set ? */
    if (!uid_default && !gid_default) {
	fprintf(stderr, "No valid uid/gid defined in the configuration file '%s'. Goodbye.\n", file);
	exit(-1);
    }

    _syslog = (slog_flty | slog_lvl);

    fclose(fd); /* All variables are init. We don't need the conf file descriptor anymore. */

    return(1);
}
